//RESTROE3 -   cont version of Restroe2, GV2002-2007
//Orbital elements <-> Orbital Elements of TwoBody Configuration
//Restroe3 was created at 12/2/2007 


#ifndef _RESTRICTED_OE
#define _RESTRICTED_OE


#ifndef pi
#define pi2 6.283185307179586477
#define pi  3.141592653589793238
#endif


//Variables for possible external use
extern double mBody,mSun,A_metric;
extern int MODEPIPI, BARYCENTRICDATA; 



typedef struct OrbitalElements
{
	double a;
	double e;
	double i;
	double M;
	double omega;
	double Omega;
	double omegabar;
	double f;
	double tau;
	double E;
	double lamda;
	double meanmotion;
	int modedisplay;  //1:display mode 0:calcmode
} OEPOS;

typedef struct OrbitalElementsDefined
{
	double a;
	double e;
	double i;
	double elpos; //pos value on ellipsis (E, M, f, lambda or tau according to "inputangle" index) 
	double omega;
	double Omega;	
	int	inputangle;  //elpos is 0:E, 1=M, 2=f, 3=lambda 4:tau
	int modedisplay; //1:display mode (deg, a in a.u.) 0:calcmode 
} OEDEF;



//============================================================
void InitRestroe(int planar, double mbody, double msun, int barycentricdata, double ametric, int modepipi);
void InitRestroeB(int planar, double mbody, double msun);
void InitRestroeH(int planar, double mbody, double msun);
void InitRestroeDef(int planar, double ametric);

//==========GLOBALLY APPLICABLE FUNCTIONS=====================
void GetOrbitalElements(double x[], OEPOS *q, int DISPLAYVALUES);
void GetInertialFromOrbital(OEDEF q, double x[]);

void OEPOStoOEDEF(OEPOS qo, OEDEF *qd);
void OEDEFtoOEPOS(OEDEF qd, OEPOS *qo);
OEDEF oedef(OEPOS qo); //direct

void BarycentricToHeliocentric(double x[]);
void HeliocentricToBarycentric(double x[]);

//-----------auxiliary ---------------------------------------------
double MeanMotion(double a);
double GetEccentricFromTrue(double f, double e);  //rad
double GetTrueFromEccentric(double E, double e); //rad
double GetMeanFromTrue(double f, double e);  //rad
double GetEccentricFromM(double M,double e); //M in rad, Newton-Raphson


void OE_DisplayValues(OEPOS *q);  //turn to deg units and a in au
void OE_CalcValues(OEPOS *q); //turn to rad units and a in nodim units
void OE_DisplayValues(OEDEF *q);  //turn to deg units and a in au
void OE_CalcValues(OEDEF *q); //turn to rad units and a in nodim units

void MPI_PI(double *x); //angle range in (-pi,pi), input=rad
void M180_180(double *x);//angle range in (-180,180), input=deg

void SwitchToSystemDimension(int direction, double x[],double xa[]);

//-------------TOOLS---------------------------------------
void fixw(double *x, int mode_deg, int mode_180_pi);







/*
typedef struct ResonantVariables
{
	double S;
	double N;
	double Sz;
	double ws;
	double wn;
	double wz;	
} RVAR;

*/

#endif