//DSYSTEM :  NBS project
//*************************** N planets + Sun ********************
//
//7/3/2017


#include <nbody\dgdsysNBs.h>
void dSystem2NBSV(double t, double y[], double dd[]);

//---------System's main variables--------------------------------
//X[0]=t,  X[1]=Data, X[2]=y1, X[3]=z1, X[4]=x2, ................, X[NDOF]=z_NB
//X[NDOF+1]=ksi1, X[NDOF+2]=ksi2, .................., X[2NDOF]=ksi_NDOF   //variations
//X[2*NDOF+1]=Datadot, X[2*NDOF+2]=y1dot, ............, X[2*NDOF+NDOF]=z_NBdot
//X[3*NDOF+1]=ksi1dot, .............., X[4*NDOF]=ksi_NDOFdot



//#define NB 2  //number of bodies (not include S-BODY) 
//#define NDOF 6 //number of Degrees of freedom (NBx3)
//#define NEQ 25 //number of 1st order deqs+1 = NDOFx4+1 //includes variational equations 
int NB, NDOF, NEQ;

double *mas, *mas0, mtot; //masses of bodies //0 pos stands for mass of SUN
double *rSi3, *rSi5, *DfSi;
double **Dkij[3], **rBij3, **rBij5, **pDerivij;
int **ss;


int InitDSys(int Nbodies, double m[], double Dt0)
{
	NB = Nbodies;
	NDOF = NB * 3;
	NEQ = NDOF * 2 + 1; //????
	mas = new double[NB + 1];		//masses  [mas[0]=mass of S-BODY
	mas0 = new double[NB + 1];		//  mas[0]+mas[i],  0: useless
	DfSi = new double[NDOF + 1];			//Variable diferences from S-Body  (XS-Xi)
											//--------------------------------------------------------------------------------
											//DfBij=Alloc2Dspace(NDOF+1,NDOF+1);	//Variable differences among bodies (Xi-Xj)
	for (int k = 0; k<3; k++) Dkij[k] = Alloc2Dspace(NB + 1, NB + 1); //Difs of k-var between i and j body e.g. D(1,2,4)=Y2-Y4
																	  //--------------------------------------------------------------------------------
	rSi3 = new double[NB + 1];		    //distances of bodies from S-BODY ^3
	rBij3 = Alloc2Dspace(NB + 1, NB + 1);   //distances between bodies ^3
	rSi5 = new double[NB + 1];		    //distances of bodies from S-BODY ^5
	rBij5 = Alloc2Dspace(NB + 1, NB + 1);   //distances between bodies ^5
	if (Dkij == NULL || DfSi == NULL || rBij3 == NULL || rBij5 == NULL || mas == NULL || rSi3 == NULL || rSi5 == NULL) return 0;
	mtot = 0;
	//interaction configuration
	ss = Alloc2DspaceIntegers(NB + 1, NB + 1);
	for (int i = 1; i <= NB; i++)	for (int j = 1; j <= NB; j++) ss[i][j] = 1; //all
	for (int i = 1; i <= NB; i++) ss[i][i] = 0; //except same bodies
	for (int i = 0; i <= NB; i++) { mas[i] = m[i]; mas0[i] = m[0] + m[i]; mtot += m[i]; }
	for (int k = 0; k<3; k++) for (int i = 0; i <= NB; i++) for (int j = 0; j <= NB; j++) Dkij[k][i][j] = 0;
	for (int i = 1; i <= NB; i++) { rBij3[i][i] = 1; rBij5[i][i] = 1; } //zero distances between the same bodies - unused nonzero values

																		//pDerivij=Alloc2Dspace(NDOF+1,NDOF+1);	//partial derivatives of coeffs of variational equations  ????
																		//if(pDerivij==NULL) return 0;

	derivs = dSystem2NBSV;
	if (!InitSI(NEQ, Dt0)) return 0; //InitBs2(NEQ,bs)
	return 1;
}

void FreeDsys()
{
	CloseSI();//CloseBs2();
	delete[] mas;
	delete[] rSi3;
	delete[] rSi5;
	delete[] DfSi;
	Delete2Dspace(rBij3, NB + 1);
	Delete2Dspace(rBij5, NB + 1);
	//Delete2Dspace(DfBij,NB+1);
	Delete2Dspace(ss, NB + 1);
	Delete2Dspace(pDerivij, NB + 1);
	for (int k = 0; k<3; k++) Delete2Dspace(Dkij[k], NB + 1);
}

//============== INDICES ==========================================
// Body : 1,2,....,NB //use as index small letter
// Variable : 0,1,2 = x,y,z

// System Position (based on NEQ) (Use as index CAPITAL letter)
//    A. Positions (1-3*NB) :Data,y1,z1, x2,y2,z2 ..., xNB, yNB,zNB, 
//    B. Variations (3*NB+1 - 6*NB): ksi1, eta1, zeta1,  ksi2,eta2,zeta2, .... ksiNB, etaNB, zetaNB
//    C. (6NB+1 - 9NB) : velocities vData,vy1,vz1, vx2,vy2,vz2, .... vxNB, vyNB, vzNB
//    D. (9NB+1 - 12*NB) : derivs of variations  vksi1, veta1, vzeta1, ....


inline int SPosIndex(int ibody, int ivar)
{
	return 3 * ibody - 2 + ivar;   //3*(ibody-1)+ivar+1
}
inline int SVelIndex(int ibody, int ivar) //Changed
{
	return 3 * ibody - 2 + ivar + NDOF;
}
inline int BodyIndex(int sposindex)
{
	return (sposindex - 1) / 3 + 1;
}
inline int VarIndex(int sposindex)
{
	return (sposindex - 1) % 3;
}
inline int BaseSposIndex(int sposindex)
{
	return sposindex - (sposindex - 1) % 3;
}
inline int dk(int i, int j)  //Kroneker's delta
{
	//if (i == j) return 1; else return 0;
	return i == j;
}

//============================================================


void dSystem2NBSV(double t, double X[], double dd[]) //Changed
{

	//S-BODY position;
	if (!CMFREE)
	{
		double XS[3]; //X,Y,Z
		for (int k = 0; k<3; k++)
		{
			double sum = 0;
			for (int i = 1; i <= NB; i++) sum += mas[i] * X[SPosIndex(i, k)];
			XS[k] = -sum / mas[0];
		}
		for (int I = 1; I <= NDOF; I++) DfSi[I] = XS[VarIndex(I)] - X[I];  //Var differencies with respect to S-BODY
	}
	//Var differencies among bodies
	for (int k = 0; k<3; k++) for (int i = 1; i <= NB; i++) for (int j = i + 1; j <= NB; j++)
	{
		Dkij[k][i][j] = X[SPosIndex(i, k)] - X[SPosIndex(j, k)]; Dkij[k][j][i] = -Dkij[k][i][j];
	}

	//functions of distances
	//A. from S-Body
	if (!CMFREE)
		for (int i = 0; i<NB; i++)
		{
			int k = 3 * i + 1;
			double r2 = DfSi[k] * DfSi[k] + DfSi[k + 1] * DfSi[k + 1] + DfSi[k + 2] * DfSi[k + 2]; //square
			rSi5[i + 1] = 1.0 / (r2*r2*sqrt(r2)); // 1/r^5
			rSi3[i + 1] = rSi5[i + 1] * r2;            // 1/r^3
		}
	//B. between bodies 
	for (int i = 1; i<NB; i++)
	{
		for (int j = i + 1; j <= NB; j++)
		{
			double r2 = Dkij[0][i][j] * Dkij[0][i][j] + Dkij[1][i][j] * Dkij[1][i][j] + Dkij[2][i][j] * Dkij[2][i][j];
			double r5 = 1.0 / (r2*r2*sqrt(r2));
			rBij5[i][j] = ss[i][j] * r5;     rBij5[j][i] = ss[j][i] * r5;  //sij/r^5
			rBij3[i][j] = rBij5[i][j] * r2;  rBij3[j][i] = rBij5[j][i] * r2;      //sij*mi/r^3
		}
	}
	//**************************************************************
	//Differential canonical equations of 2ND ORDER [I=1 ... NDOF]
	//**************************************************************
	for (int I = 0; I <= 2 * NDOF; I++) dd[I] = 0; // Initialize derivs
	if (!CMFREE)
	{
		for (int I = 1; I <= NDOF; I++) dd[I] = mas[0] * rSi3[BodyIndex(I)] * DfSi[I]; // (mo/r^3)*(x0-xi)
	}
	//add internal_interactions
	for (int I = 1; I <= NDOF; I++) {
		int i = BodyIndex(I); int v = VarIndex(I);
		for (int l = 1; l <= NB; l++) dd[I] += mas[l] * rBij3[l][i] * Dkij[v][l][i];
	}

	//for(int I=1;I<=NDOF;I++) dd[I+NDOF]=0;
	//return;
	/*
	//**************************************************************
	//Coefficients of Variational equations NDOF x NDOF   partialX[I]/partial X[J]
	//**************************************************************
	//first part (M1) - S-BODY terms
	for(int I=1;I<=NDOF;I++){
	int Ib=BaseSposIndex(I);
	int i=BodyIndex(I);
	int vi = VarIndex(I);
	//double f1=3*rSi5[i]*DfSi[I];
	for (int J = 1; J <= NDOF; J++) {
	int Jb = BaseSposIndex(J);
	int j = BodyIndex(J);
	int vj = VarIndex(J);
	pDerivij[I][J] = 3*(mas[j] + mas[0] * dk(i, j))*rSi5[i] * DfSi[Ib + vj] * DfSi[I] - dk(vi, vj)*rSi3[i]*(mas[j]+mas[0]*dk(I,J));
	}
	}
	*/
	/*printf("------------------------------\n");
	printf("m1=%f   m2=%f   m0=%f\n", mas[1], mas[2], mas[0]);
	printf("Data=%f   y1=%f   x2=%f   y2=%f\n", X[1], X[2], X[4], X[5]);
	int I = 1;  for (int J = 1; J <= NDOF; J++) printf("[%d][%d]  %f\n", I, J, pDerivij[I][J]);
	getchar();*/
	/*
	//second part (M2) - internal interaction terms
	for (int I = 1; I <= NDOF; I++){
	int Ib = BaseSposIndex(I);
	int i = BodyIndex(I);
	int vi = VarIndex(I);
	for (int J = 1; J <= NDOF; J++){
	int Jb = BaseSposIndex(J);
	int j = BodyIndex(J);
	int vj = VarIndex(J);
	double sum = 0.0;
	if (i == j)
	for (int l = 1; l <= NB; l++)
	sum += mas[l] * (3.0 * rBij5[l][i] * Dkij[vj][l][i] * Dkij[vi][l][i] + (dk(SPosIndex(l, vi), J) - dk(I, J))*rBij3[l][i]);
	else sum = -mas[j] * (3.0 * rBij5[j][i] * Dkij[vj][j][i] * Dkij[vi][j][i] - dk(vi, vj)*rBij3[j][i]);
	pDerivij[I][J] += sum;
	}
	}
	*/
	/*printf("------------------------------\n");
	printf("m1=%f   m2=%f   m0=%f\n", mas[1], mas[2], mas[0]);
	printf("Data=%f   y1=%f   x2=%f   y2=%f\n", X[1], X[2], X[4], X[5]);
	I = 1;  for (int J = 1; J <= NDOF; J++) printf("[%d][%d]  %f\n", I, J, pDerivij[I][J]);
	getchar();*/
	/*
	//**************************************************************
	//Variational equations NDOF
	//**************************************************************
	for(int I=1;I<=NDOF;I++){
	double sum=0;
	for(int J=1;J<=NDOF;J++) sum+=pDerivij[I][J]*X[J+NDOF];
	dd[I+NDOF]=sum;
	//printf("%d    %e\n", I + NDOF, dd[I + NDOF]);
	}
	*/
}



void InitVariations0(double X0[], int mode)
{
	for (int i = NDOF + 1; i <= 2 * NDOF; i++) X0[i] = 0;
	for (int i = 3 * NDOF + 1; i<NEQ; i++) X0[i] = 0;
	switch (mode) {
	case 1: X0[NDOF + 1] = 1; break;
	case 2: for (int k = 0; k<NB; k++) X0[NDOF + 3 * k + 1] = 1 / sqrt(1.0*NB); break;
	}
}


//=========S-BODY, DISTANCES and FORCES =======================================


void SBodyPos(double X[], double XS[]) //in X, out XS: external array for Sun pos/vel  //Changed
{
	//pos
	for (int j = 0; j<3; j++) {
		double sum = 0;
		for (int i = 1; i <= NB; i++) sum += mas[i] * X[SPosIndex(i, j)];
		XS[j + 1] = -sum / mas[0];
	}
	//vel
	for (int j = 0; j<3; j++) {
		double sum = 0;
		for (int i = 1; i <= NB; i++) sum += mas[i] * X[SPosIndex(i, j) + NDOF];
		XS[j + 4] = -sum / mas[0];
	}
}


double Sdistance(double XS[], double X[], int i)
{
	int I = SPosIndex(i, 0);
	double r = (XS[1] - X[I])*(XS[1] - X[I]) + (XS[2] - X[I + 1])*(XS[2] - X[I + 1]) + (XS[3] - X[I + 2])*(XS[3] - X[I + 2]);
	return sqrt(r);
}

double BodiesDistance(double X[], int i, int j)
{
	int I = SPosIndex(i, 0); int J = SPosIndex(j, 0);
	double r = 0;
	for (int k = 0; k<3; k++) r += (X[I + k] - X[J + k])*(X[I + k] - X[J + k]);
	return sqrt(r);
}


//==========================================================================
//                      Integrals IN THE INERTIAL FRAME
//==========================================================================


//A. Angular Momentum


void AngularMomentumBody(int i, double X[], double *Lx, double *Ly, double *Lz)
{
	double x = X[SPosIndex(i, 0)]; double xd = X[SVelIndex(i, 0)];
	double y = X[SPosIndex(i, 1)]; double yd = X[SVelIndex(i, 1)];
	double z = X[SPosIndex(i, 2)]; double zd = X[SVelIndex(i, 2)];
	*Lx = mas[i] * (y*zd - yd*z);
	*Ly = mas[i] * (z*xd - zd*x);
	*Lz = mas[i] * (x*yd - xd*y);
}


double AngularMomentum(double X[], double *Lx, double *Ly, double *Lz)
{
	double lx, ly, lz;
	*Lx = *Ly = *Lz = 0.0;
	for (int i = 1; i <= NB; i++)
	{
		AngularMomentumBody(i, X, &lx, &ly, &lz);
		*Lx += lx; *Ly += ly; *Lz += lz;
	}
	//S-BODY
	if (!CMFREE)
	{
		double XS[7];
		SBodyPos(X, XS);
		lz = mas[0] * (XS[1] * XS[5] - XS[2] * XS[4]);
		lx = mas[0] * (XS[2] * XS[6] - XS[5] * XS[3]);
		ly = mas[0] * (XS[3] * XS[4] - XS[6] * XS[1]);
		*Lx += lx; *Ly += ly; *Lz += lz;
	}
	return sqrt((*Lx)*(*Lx) + (*Ly)*(*Ly) + (*Lz)*(*Lz));
}

double AngularMomentum(double X[]) //as above 
{
	double lx, ly, lz;
	double Lx, Ly, Lz;  Lx = Ly = Lz = 0.0;
	for (int i = 1; i <= NB; i++) { AngularMomentumBody(i, X, &lx, &ly, &lz); Lx += lx; Ly += ly; Lz += lz; }
	if (!CMFREE)
	{
		double XS[7];
		SBodyPos(X, XS);
		lz = mas[0] * (XS[1] * XS[5] - XS[2] * XS[4]); lx = mas[0] * (XS[2] * XS[6] - XS[5] * XS[3]); ly = mas[0] * (XS[3] * XS[4] - XS[6] * XS[1]);
		Lx += lx; Ly += ly; Lz += lz;
	}
	return sqrt(Lx*Lx + Ly*Ly + Lz*Lz);
}


//B. Energy

double KineticEnergyBody(int i, double X[])
{
	return 0.5*mas[i] * (X[SVelIndex(i, 0)] * X[SVelIndex(i, 0)] + X[SVelIndex(i, 1)] * X[SVelIndex(i, 1)] + X[SVelIndex(i, 2)] * X[SVelIndex(i, 2)]);
}

double KineticEnergySbody(double XS[])
{
	return 0.5*mas[0] * (XS[4] * XS[4] + XS[5] * XS[5] + XS[6] * XS[6]);
}

double PotentialSbody(double XS[], double X[])
{
	double vsbody = 0;
	for (int i = 1; i <= NB; i++) {
		double r = Sdistance(XS, X, i);
		vsbody += mas[i] / r;
	}
	return -vsbody*mas[0];
}

double PotentialBodies(double X[])
{
	double vv = 0;
	for (int i = 1; i<NB; i++) for (int j = i + 1; j <= NB; j++)
	{
		double r = BodiesDistance(X, i, j);
		vv += mas[j] * mas[i] / r;
	}
	return -vv;
}


double Energy(double X[])
{
	double TS = 0, VS = 0;
	if (!CMFREE)
	{
		double XS[7];
		SBodyPos(X, XS);
		TS = KineticEnergySbody(XS);
		VS = PotentialSbody(XS, X);
	}
	double Tb = 0; for (int i = 1; i <= NB; i++) Tb += KineticEnergyBody(i, X);
	double Vb = PotentialBodies(X);
	return TS + Tb + VS + Vb;
}


//---------Chaoticity indices----------------------

double CalcDFLI(double X[])
{
	double sum = 0;
	for (int I = NDOF + 1; I <= 2 * NDOF; I++) sum += X[I] * X[I];
	return sqrt(sum) / X[0];
}
