//Published under The MIT License (MIT)

#pragma warning (disable : 4018 )
#pragma warning (disable : 4996)   // unsafe functions
#pragma warning (disable : 4305)


#include <GL\glut.h>
#include <solar_system\tga.h>
#include <solar_system\camera.h>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <vector>



#include <nbody\dgdsysNBs.h>
#include <nbody\giroeNBs.h>
#include <nbody\gTools.h>
#include <nbody\Symp6s.h>
#include <nbody\restroe3.h>
#include <fstream>
#include <conio.h>
#include <nbody\drun0.h>
#include <iostream>


#define NMAXPOINTS 100000

using namespace std;

double *radius;


extern double dt; 
extern OEDEF oeb[NMAXBODIES];
extern int Nsteps;
extern int NB, NDOF, NEQ;

double *X0 , *X ,*Y;
							         
double Data[NMAXPOINTS][1000]; 
int NcurData;

double t;


// toggles if orbits are drawn
bool showOrbits = true;


void file_input();
void file_change(char system_name[]);

void Symplectic6thRUN(double yin[], int n, double dt, double yout[]);

void Symplectic_translate();

// the screen size
int screenWidth, screenHeight;

// The instance of the camera
Camera camera;


// holds the state of the controls for the camera - when true, the key for that control is being pressed
struct ControlStates
{
	bool forward, backward, left, right, yawLeft, yawRight, pitchUp,
		pitchDown, rollLeft, rollRight;
} controls;

// The TGA texture containing the planet textures and the stars for the skybox
TGA *stars, *sun, *jupiter, *mars, *mercury, *moon, *neptune, *pluto, *saturn, *uranus, *venus;



void myMenu(int n)
{
	switch (n) {
	case 1: file_change("HD106315"); 
		break;
	case 2: file_change("kepler-9");
		break;
	case 3: file_change("gliese876");
		break;
	case 4: file_change("Sun");
		break;
	case 5: file_change("unstable");
		break;
	case 6: file_change("user_program");
		break;
	case 7: exit(0);
	}
	X0 = new double[NEQ];

	GetSystemPosV(oeb, X0);

	X = new double[NEQ];
	Y = new double[NEQ];

	for (int i = 0; i < NEQ; i++)
	{
		X[i] = X0[i];
	}
	t = X[0] * t_metric;

	NcurData = 0;

	float vec[3];
	vec[0] = 0;
	vec[1] = 0;
	vec[2] = 0;

	camera.pointAt(vec);

	glutPostRedisplay();
}



void Symplectic_translate()
{
	
	Symplectic6thRUN(X, Nsteps, dt, Y);
	t = Y[0]; // *t_metric;

	
	for (int i = 0; i < NEQ; i++)
	{
		Data[NcurData][i] = X[i];
		X[i] = Y[i];
	}
	NcurData++;
	 //double dE = Ener0 - Energy(X);
	// printf("dE=%d\n", dE);
	// double dL = Lang0 - AngularMomentum(X);

	return;
}

// timer function called every 10ms or more
void timer(int)
{
	glutPostRedisplay(); // post for display func
	glutTimerFunc(10, timer, 0); // limit frame drawing to 100fps
}


void init(void)
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_TEXTURE_2D);

	// set up lighting
	glEnable(GL_LIGHTING);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	GLfloat matSpecular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat matAmbience[] = { 0.3, 0.3, 0.3, 1.0 };
	GLfloat matShininess[] = { 20.0 };
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_SMOOTH);

	glMaterialfv(GL_FRONT, GL_SPECULAR, matSpecular);
	glMaterialfv(GL_FRONT, GL_SHININESS, matShininess);
	glMaterialfv(GL_FRONT, GL_AMBIENT, matAmbience);

	GLfloat lightAmbient[] = { 0.3, 0.3, 0.3, 1.0 };
	GLfloat lightDiffuse[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat lightSpecular[] = { 1.0, 1.0, 1.0, 1.0 };

	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glDisable(GL_LIGHTING);

	stars = new TGA("images/stars.tga");
	sun = new TGA("images/sun.tga");    
	jupiter = new TGA("images/jupiter.tga");
	mars = new TGA("images/mars.tga");
	mercury = new TGA("images/mercury.tga");
	moon = new TGA("images/moon.tga");
	neptune = new TGA("images/neptune.tga");
	pluto = new TGA("images/pluto.tga");
	saturn = new TGA("images/saturn.tga");
	uranus = new TGA("images/uranus.tga");
	venus = new TGA("images/venus.tga");
	


	// reset controls
	controls.forward = false;
	controls.backward = false;
	controls.left = false;
	controls.right = false;
	controls.rollRight = false;
	controls.rollLeft = false;
	controls.pitchDown = false;
	controls.pitchUp = false;
	controls.yawLeft = false;
	controls.yawRight = false;


	timer(0);
}

void drawCube(void);

void display(void)
{
	

	if (controls.forward) camera.forward();		if (controls.backward) camera.backward();
	if (controls.left) camera.left();			if (controls.right) camera.right();
	if (controls.yawLeft) camera.yawLeft();		if (controls.yawRight) camera.yawRight();
	if (controls.rollLeft) camera.rollLeft();	if (controls.rollRight) camera.rollRight();
	if (controls.pitchUp) camera.pitchUp();		if (controls.pitchDown) camera.pitchDown();


	// clear the buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glColor3f(1.0, 1.0, 1.0);


	// set up the perspective matrix for rendering the 3d world
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(70.0f, (float)screenWidth / (float)screenHeight, 0.001f, 500.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();


	// perform the camera orientation transform
	camera.transformOrientation();


	// draw the skybox
	glBindTexture(GL_TEXTURE_2D, stars->getTextureHandle());
	drawCube();


	// perform the camera translation transform
	camera.transformTranslation();


	GLfloat lightPosition[] = { 0.0, 0.0, 0.0, 1.0 };
	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);


	// render the solar system
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);


	// render as a GLU sphere quadric object
	GLUquadricObj* quadric = gluNewQuadric();
	gluQuadricTexture(quadric, true);
	gluQuadricNormals(quadric, GLU_SMOOTH);


	Symplectic_translate();

	glBindTexture(GL_TEXTURE_2D, sun->getTextureHandle());

	glPushMatrix();
	glTranslatef(0, 0, 0);
	glRotatef(Y[0]*10, 0.0f, 0.0f, 1.0f);
	glDisable(GL_LIGHTING);
	gluSphere(quadric, 0.2, 30, 30);   // sun
	glEnable(GL_LIGHTING);
	glPopMatrix();


	int j;
	j = 0;

	for (int i = 1; i <= NB; i++)
	{
		if (i == 1)
		{
			glBindTexture(GL_TEXTURE_2D, venus->getTextureHandle());
		}

		if (i == 2)
		{
			glBindTexture(GL_TEXTURE_2D, pluto->getTextureHandle());
		}

		if (i == 3)
		{
			glBindTexture(GL_TEXTURE_2D, jupiter->getTextureHandle());
		}

		if (i == 4)
		{
			glBindTexture(GL_TEXTURE_2D, mars->getTextureHandle());
		}

		if (i == 5)
		{
			glBindTexture(GL_TEXTURE_2D, saturn->getTextureHandle());
		}

		if (i > 5)
		{
			glBindTexture(GL_TEXTURE_2D, moon->getTextureHandle());
		}

		glPushMatrix();
		glTranslatef(Y[1 + j], Y[2 + j], Y[3 + j]);
		gluSphere(quadric, radius[i], 30, 30);
		glPopMatrix();
		j += 3;

	}
	

	if (showOrbits)
		{
			glDisable(GL_LIGHTING);

			int o = 0;

			for (int i = 0; i < NB; i++)
				{
					glBegin(GL_POINTS);
					
					for (int j = NcurData - 1; j > 0; j--)
						{
	
							glVertex3f(Data[j][o + 1], Data[j][o + 2], Data[j][o + 3]);
	
						}

					o += 3;

					glEnd();

				}
			glEnable(GL_LIGHTING);

		}


	glDisable(GL_LIGHTING);

	glDisable(GL_DEPTH_TEST);


	glFlush();
	glutSwapBuffers();
}

void keyDown(unsigned char key, int x, int y)
{
	
	
	switch (key)
	{

	case 'o':
		showOrbits = !showOrbits; // toggle show orbits
		break;

	case ',':
		camera.slowDown(); // slow down camera
		break;
	case '.':
		camera.speedUp(); // speed up camera
		break;

		// these are all camera controls
	case 'w':
		controls.forward = true;
		break;
	case 's':
		controls.backward = true;
		break;
	case 'a':
		controls.left = true;
		break;
	case 'd':
		controls.right = true;
		break;
	case 'l':
		controls.rollRight = true;
		break;
	case 'j':
		controls.rollLeft = true;
		break;
	case 'i':
		controls.pitchDown = true;
		break;
	case 'k':
		controls.pitchUp = true;
		break;
	case 'q':
		controls.yawLeft = true;
		break;
	case 'e':
		controls.yawRight = true;
		break;
	}

}

void keyUp(unsigned char key, int x, int y)
{
	switch (key)
	{
	case 'w':
		controls.forward = false;
		break;
	case 's':
		controls.backward = false;
		break;
	case 'a':
		controls.left = false;
		break;
	case 'd':
		controls.right = false;
		break;
	case 'l':
		controls.rollRight = false;
		break;
	case 'j':
		controls.rollLeft = false;
		break;
	case 'i':
		controls.pitchDown = false;
		break;
	case 'k':
		controls.pitchUp = false;
		break;
	case 'q':
		controls.yawLeft = false;
		break;
	case 'e':
		controls.yawRight = false;
		break;
	}
}

void reshape(int w, int h)
{
	screenWidth = w;
	screenHeight = h;
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
}


void SetupMenu()
{
	glutCreateMenu(myMenu);
	glutAddMenuEntry("HD 106315", 1);
	glutAddMenuEntry("kepler-9", 2);
	glutAddMenuEntry("gliese876", 3);
	glutAddMenuEntry("Sun", 4);
	glutAddMenuEntry("unstable", 5);
	glutAddMenuEntry("user_system", 6);
	glutAddMenuEntry("exit", 7);
	glutAttachMenu(GLUT_RIGHT_BUTTON);
}


int main(int argc, char** argv)
{
	printf("Use the camera with the keys: w,s,a,d for forward,backward,left and right movement respectively\n");
	printf("The keys: q,e,j,k,l,i,k rotate the camera in different directions\n");
	
	radius = new double[100];
	file_input();

	X0 = new double[NEQ];
	
	GetSystemPosV(oeb, X0); 

	NcurData = 0;

	X = new double[NEQ];
	Y = new double[NEQ];

	for (int i = 0; i < NEQ; i++)
	{
		X[i] = X0[i];
	}
	double t = X[0] * t_metric;


	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(1200, 700);
	glutInitWindowPosition(100, 100);
	glutCreateWindow(argv[0]);    
	init();
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyDown); 
	glutKeyboardUpFunc(keyUp);
	SetupMenu();
	

	glutMainLoop();
	
	return 0;
}

void drawCube(void)
{
	glBegin(GL_QUADS);
	// new face
	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-1.0f, -1.0f, 1.0f);
	glTexCoord2f(1.0f, 0.0f);	glVertex3f(1.0f, -1.0f, 1.0f);
	glTexCoord2f(1.0f, 1.0f);	glVertex3f(1.0f, 1.0f, 1.0f);
	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-1.0f, 1.0f, 1.0f);
	// new face
	glTexCoord2f(0.0f, 0.0f);	glVertex3f(1.0f, 1.0f, 1.0f);
	glTexCoord2f(1.0f, 0.0f);	glVertex3f(1.0f, 1.0f, -1.0f);
	glTexCoord2f(1.0f, 1.0f);	glVertex3f(1.0f, -1.0f, -1.0f);
	glTexCoord2f(0.0f, 1.0f);	glVertex3f(1.0f, -1.0f, 1.0f);
	// new face
	glTexCoord2f(0.0f, 0.0f);	glVertex3f(1.0f, 1.0f, -1.0f);
	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-1.0f, 1.0f, -1.0f);
	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-1.0f, -1.0f, -1.0f);
	glTexCoord2f(0.0f, 1.0f);	glVertex3f(1.0f, -1.0f, -1.0f);
	// new face
	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-1.0f, -1.0f, -1.0f);
	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-1.0f, -1.0f, 1.0f);
	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-1.0f, 1.0f, 1.0f);
	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-1.0f, 1.0f, -1.0f);
	// new face
	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-1.0f, 1.0f, -1.0f);
	glTexCoord2f(1.0f, 0.0f);	glVertex3f(1.0f, 1.0f, -1.0f);
	glTexCoord2f(1.0f, 1.0f);	glVertex3f(1.0f, 1.0f, 1.0f);
	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-1.0f, 1.0f, 1.0f);
	// new face
	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-1.0f, -1.0f, -1.0f);
	glTexCoord2f(1.0f, 0.0f);	glVertex3f(1.0f, -1.0f, -1.0f);
	glTexCoord2f(1.0f, 1.0f);	glVertex3f(1.0f, -1.0f, 1.0f);
	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-1.0f, -1.0f, 1.0f);

	glEnd();
}
